@@ -8,9 +8,12 @@ use std::convert::TryFrom;
 macro_rules! apply {
     ($self:expr, $f:expr) => {{
         if $self.null_count() == 0 {
-            $self.into_no_null_iter().map($f).collect()
+            $self.into_no_null_iter().map($f).collect_trusted()
         } else {
-            $self.into_iter().map(|opt_v| opt_v.map($f)).collect()
+            $self
+                .into_iter()
+                .map(|opt_v| opt_v.map($f))
+                .collect_trusted()
         }
     }};
 }
@@ -18,13 +21,17 @@ macro_rules! apply {
 macro_rules! apply_enumerate {
     ($self:expr, $f:expr) => {{
         if $self.null_count() == 0 {
-            $self.into_no_null_iter().enumerate().map($f).collect()
+            $self
+                .into_no_null_iter()
+                .enumerate()
+                .map($f)
+                .collect_trusted()
         } else {
             $self
                 .into_iter()
                 .enumerate()
                 .map(|(idx, opt_v)| opt_v.map(|v| $f((idx, v))))
-                .collect()
+                .collect_trusted()
         }
     }};
 }
@@ -42,7 +49,7 @@ where
             .data_views()
             .zip(self.null_bits())
             .map(|(slice, (_null_count, opt_buffer))| {
-                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect();
+                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect_trusted();
                 (vec, opt_buffer.cloned())
             })
             .collect();
@@ -59,9 +66,18 @@ where
             .downcast_iter()
             .map(|array| {
                 let av: AlignedVec<_> = if array.null_count() == 0 {
-                    array.values().iter().map(|&v| f(Some(v))).collect()
+                    array
+                        .values()
+                        .iter()
+                        .map(|&v| f(Some(v)))
+                        .trust_my_length(array.len())
+                        .collect_trusted()
                 } else {
-                    array.into_iter().map(f).collect()
+                    array
+                        .into_iter()
+                        .map(f)
+                        .trust_my_length(array.len())
+                        .collect_trusted()
                 };
                 Arc::new(av.into_primitive_array::<S>(None)) as ArrayRef
             })
@@ -78,7 +94,7 @@ where
             .into_iter()
             .zip(self.null_bits())
             .map(|(slice, (_null_count, opt_buffer))| {
-                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect();
+                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect_trusted();
                 (vec, opt_buffer.cloned())
             })
             .collect();
@@ -90,11 +106,17 @@ where
     where
         F: Fn(Option<T::Native>) -> Option<T::Native> + Copy,
     {
-        self.downcast_iter()
-            .flatten()
-            .trust_my_length(self.len())
-            .map(f)
-            .collect()
+        let chunks = self
+            .downcast_iter()
+            .map(|arr| {
+                let iter = arr.into_iter().map(f);
+                // Safety:
+                // array's length is trusted
+                let arr = unsafe { PrimitiveArray::<T>::from_trusted_len_iter(iter) };
+                Arc::new(arr) as ArrayRef
+            })
+            .collect();
+        Self::new_from_chunks(self.name(), chunks)
     }
 
     fn apply_with_idx<F>(&'a self, f: F) -> Self
@@ -102,7 +124,11 @@ where
         F: Fn((usize, T::Native)) -> T::Native + Copy,
     {
         if self.null_count() == 0 {
-            let ca: NoNull<_> = self.into_no_null_iter().enumerate().map(f).collect();
+            let ca: NoNull<_> = self
+                .into_no_null_iter()
+                .enumerate()
+                .map(f)
+                .collect_trusted();
             ca.into_inner()
         } else {
             self.downcast_iter()
@@ -110,7 +136,7 @@ where
                 .trust_my_length(self.len())
                 .enumerate()
                 .map(|(idx, opt_v)| opt_v.map(|v| f((idx, v))))
-                .collect()
+                .collect_trusted()
         }
     }
 
@@ -123,7 +149,7 @@ where
             .trust_my_length(self.len())
             .enumerate()
             .map(f)
-            .collect()
+            .collect_trusted()
     }
     fn apply_to_slice<F, V>(&'a self, f: F, slice: &mut [V])
     where
@@ -153,7 +179,7 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
         self.apply_kernel_cast(|array| {
             let av: AlignedVec<_> = (0..array.len())
                 .map(|idx| unsafe { f(array.value_unchecked(idx)) })
-                .collect();
+                .collect_trusted();
             let null_bit_buffer = array.data_ref().null_buffer().cloned();
             Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
         })
@@ -165,7 +191,11 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
         S: PolarsNumericType,
     {
         self.apply_kernel_cast(|array| {
-            let av: AlignedVec<_> = array.into_iter().map(f).collect();
+            let av: AlignedVec<_> = array
+                .into_iter()
+                .map(f)
+                .trust_my_length(array.len())
+                .collect_trusted();
             Arc::new(av.into_primitive_array::<S>(None)) as ArrayRef
         })
     }
@@ -181,7 +211,7 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
     where
         F: Fn(Option<bool>) -> Option<bool> + Copy,
     {
-        self.into_iter().map(f).collect()
+        self.into_iter().map(f).collect_trusted()
     }
 
     fn apply_with_idx<F>(&'a self, f: F) -> Self
@@ -195,7 +225,7 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
     where
         F: Fn((usize, Option<bool>)) -> Option<bool> + Copy,
     {
-        self.into_iter().enumerate().map(f).collect()
+        self.into_iter().enumerate().map(f).collect_trusted()
     }
 
     fn apply_to_slice<F, T>(&'a self, f: F, slice: &mut [T])
@@ -229,7 +259,7 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
             .map(|array| {
                 let av: AlignedVec<_> = (0..array.len())
                     .map(|idx| unsafe { f(array.value_unchecked(idx)) })
-                    .collect();
+                    .collect_trusted();
                 let null_bit_buffer = array.data_ref().null_buffer().cloned();
                 Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
             })
@@ -246,7 +276,11 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
             .downcast_iter()
             .into_iter()
             .map(|array| {
-                let av: AlignedVec<_> = array.into_iter().map(f).collect();
+                let av: AlignedVec<_> = array
+                    .into_iter()
+                    .map(f)
+                    .trust_my_length(array.len())
+                    .collect_trusted();
                 let null_bit_buffer = array.data_ref().null_buffer().cloned();
                 Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
             })
@@ -265,7 +299,9 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
     where
         F: Fn(Option<&'a str>) -> Option<Cow<'a, str>> + Copy,
     {
-        self.into_iter().map(f).collect()
+        let mut ca: Self = self.into_iter().map(f).collect_trusted();
+        ca.rename(self.name());
+        ca
     }
 
     fn apply_with_idx<F>(&'a self, f: F) -> Self
@@ -279,7 +315,9 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
     where
         F: Fn((usize, Option<&'a str>)) -> Option<Cow<'a, str>> + Copy,
     {
-        self.into_iter().enumerate().map(f).collect()
+        let mut ca: Self = self.into_iter().enumerate().map(f).collect_trusted();
+        ca.rename(self.name());
+        ca
     }
 
     fn apply_to_slice<F, T>(&'a self, f: F, slice: &mut [T])
@@ -382,7 +420,7 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
                         let series = Series::try_from(("", arrayref)).unwrap();
                         f(series)
                     })
-                    .collect();
+                    .collect_trusted();
                 let null_bit_buffer = array.data_ref().null_buffer().cloned();
                 Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
             })
@@ -411,7 +449,7 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
 
                         f(v)
                     })
-                    .collect();
+                    .collect_trusted();
                 let null_bit_buffer = array.data_ref().null_buffer().cloned();
                 Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
             })
@@ -431,7 +469,7 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
     where
         F: Fn(Option<Series>) -> Option<Series> + Copy,
     {
-        self.into_iter().map(f).collect()
+        self.into_iter().map(f).collect_trusted()
     }
 
     /// Apply a closure elementwise. The closure gets the index of the element as first argument.
@@ -447,7 +485,7 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
     where
         F: Fn((usize, Option<Series>)) -> Option<Series> + Copy,
     {
-        self.into_iter().enumerate().map(f).collect()
+        self.into_iter().enumerate().map(f).collect_trusted()
     }
 
     fn apply_to_slice<F, T>(&'a self, f: F, slice: &mut [T])
@@ -470,3 +508,74 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
         });
     }
 }
+
+#[cfg(feature = "object")]
+impl<'a, T> ChunkApply<'a, &'a T, T> for ObjectChunked<T>
+where
+    T: PolarsObject,
+{
+    fn apply_cast_numeric<F, S>(&'a self, _f: F) -> ChunkedArray<S>
+    where
+        F: Fn(&'a T) -> S::Native + Copy,
+        S: PolarsNumericType,
+    {
+        todo!()
+    }
+
+    fn branch_apply_cast_numeric_no_null<F, S>(&'a self, _f: F) -> ChunkedArray<S>
+    where
+        F: Fn(Option<&'a T>) -> S::Native + Copy,
+        S: PolarsNumericType,
+    {
+        todo!()
+    }
+
+    fn apply<F>(&'a self, f: F) -> Self
+    where
+        F: Fn(&'a T) -> T + Copy,
+    {
+        let mut ca: ObjectChunked<T> = self.into_iter().map(|opt_v| opt_v.map(f)).collect();
+        ca.rename(self.name());
+        ca
+    }
+
+    fn apply_on_opt<F>(&'a self, f: F) -> Self
+    where
+        F: Fn(Option<&'a T>) -> Option<T> + Copy,
+    {
+        let mut ca: ObjectChunked<T> = self.into_iter().map(f).collect();
+        ca.rename(self.name());
+        ca
+    }
+
+    fn apply_with_idx<F>(&'a self, _f: F) -> Self
+    where
+        F: Fn((usize, &'a T)) -> T + Copy,
+    {
+        todo!()
+    }
+
+    fn apply_with_idx_on_opt<F>(&'a self, _f: F) -> Self
+    where
+        F: Fn((usize, Option<&'a T>)) -> Option<T> + Copy,
+    {
+        todo!()
+    }
+
+    fn apply_to_slice<F, V>(&'a self, f: F, slice: &mut [V])
+    where
+        F: Fn(Option<&'a T>, &V) -> V,
+    {
+        assert!(slice.len() >= self.len());
+        let mut idx = 0;
+        self.downcast_iter().for_each(|arr| {
+            arr.into_iter().for_each(|opt_val| {
+                // Safety:
+                // length asserted above
+                let item = unsafe { slice.get_unchecked_mut(idx) };
+                *item = f(opt_val, item);
+                idx += 1;
+            })
+        });
+    }
+}
