@@ -1,5 +1,21 @@
 //! Everything you need to get started with Polars.
+pub use std::sync::Arc;
+
+pub use arrow::datatypes::{ArrowPrimitiveType, Field as ArrowField, Schema as ArrowSchema};
+
+pub use polars_arrow::vec::AlignedVec;
+pub(crate) use polars_arrow::{array::*, trusted_len::TrustedLen};
+
+#[cfg(feature = "object")]
+pub use crate::chunked_array::object::PolarsObject;
+pub use crate::chunked_array::ops::take::take_random::{
+    IntoTakeRandom, NumTakeRandomChunked, NumTakeRandomCont,
+};
+#[cfg(feature = "temporal")]
+pub use crate::chunked_array::temporal::conversion::*;
 pub(crate) use crate::frame::groupby::aggregations::*;
+#[cfg(feature = "checked_arithmetic")]
+pub use crate::series::arithmetic::checked::NumOpsDispatchChecked;
 pub use crate::{
     chunked_array::{
         arithmetic::Pow,
@@ -10,14 +26,7 @@ pub use crate::{
         },
         comparison::NumComp,
         iterator::{IntoNoNullIterator, PolarsIterator},
-        ops::{
-            aggregate::*,
-            chunkops::ChunkOps,
-            take::AsTakeIndex,
-            take_random::{IntoTakeRandom, NumTakeRandomChunked, NumTakeRandomCont},
-            window::InitFold,
-            *,
-        },
+        ops::{aggregate::*, chunkops::ChunkOps, window::InitFold, *},
         ChunkedArray,
     },
     datatypes,
@@ -32,14 +41,3 @@ pub use crate::{
     utils::IntoVec,
     vector_hasher::VecHash,
 };
-pub use arrow::datatypes::{ArrowPrimitiveType, Field as ArrowField, Schema as ArrowSchema};
-pub use polars_arrow::vec::AlignedVec;
-pub(crate) use polars_arrow::{array::*, trusted_len::TrustedLen};
-pub use std::sync::Arc;
-
-#[cfg(feature = "object")]
-pub use crate::chunked_array::object::PolarsObject;
-#[cfg(feature = "temporal")]
-pub use crate::chunked_array::temporal::conversion::*;
-#[cfg(feature = "checked_arithmetic")]
-pub use crate::series::arithmetic::checked::NumOpsDispatchChecked;
